@@ -2,6 +2,8 @@ $set 14 #main
 
 $ #RCRequiresArg
 # Erreur: '-rc' ncessite un argument\n
+$ #MENURequiresArg
+# Erreur: '-menu' ncessite un argument\n
 $ #DISPLAYRequiresArg
 # Erreur: '-display' ncessite un argument\n
 $ #WarnDisplaySet
@@ -12,6 +14,7 @@ $ #Usage
   \t\t\t  1997 - 2000 Brad Hughes\n\n\
   -display <chane>\t\tutilise la connexion  l'affichage.\n\
   -rc <chane>\t\t\tutilise un autre fichier de configuration.\n\
+  -menu <chane>\t\t\tutilise un autre fichier de menu.\n\
   -version\t\t\taffiche la version et quitte.\n\
   -help\t\t\t\taffiche ce texte d'aide et quitte.\n\n
 $ #CompileOptions
