@@ -2,6 +2,8 @@ $set 14 #main
 
 $ #RCRequiresArg
 # napaka: '-rc' zahteva argument\n
+$ #MENURequiresArg
+# napaka: '-menu' zahteva argument\n
 $ #DISPLAYRequiresArg
 # napaka: '-display' zahteva argument\n
 $ #WarnDisplaySet
@@ -12,6 +14,7 @@ $ #Usage
   \t\t\t  1997 - 2000 Brad Hughes\n\n\
   -display <string>\t\tuporabi prikazovalnik.\n\
   -rc <string>\t\t\tuporabi nadomestno datoteko z viri.\n\
+  -menu <string>\t\t\tuse alternate menu file.\n\
   -version\t\t\tprikai oznako razliice in konaj.\n\
   -help\t\t\t\prikai ta navodila in konaj.\n\n
 $ #CompileOptions
